/***************************************************************************
                          sq_popupmenu.h  -  description
                             -------------------
    begin                : ??? June 3 2006
    copyright            : (C) 2006 by Baryshev Dmitry
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SQ_POPUPMENU_H
#define SQ_POPUPMENU_H

#include <tdepopupmenu.h>

/**
  *@author Baryshev Dmitry
  */

class SQ_PopupMenu : public TDEPopupMenu
{
    public: 
	SQ_PopupMenu(TQWidget *parent = 0, const char *name = 0);
    	~SQ_PopupMenu();

        void insertTitle(const TQString &t);
        void changeTitle(const TQString &t);

     private:
        int title;
};

#endif
