/***************************************************************************
                          sq_mountviewitem.h  -  description
                             -------------------
    begin                : ??? Feb 24 2007
    copyright            : (C) 2007 by Baryshev Dmitry
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SQ_MOUNTVIEWITEM_H
#define SQ_MOUNTVIEWITEM_H

#include <tdelistview.h>

class SQ_MountViewItem : public TDEListViewItem
{
    public:
        SQ_MountViewItem(TDEListView *parent, const TQString &mpoint);
        ~SQ_MountViewItem();

        bool mounted() const;
        void setMounted(bool b);

        TQString device() const;
        void setDevice(const TQString &dev);

    private:
        bool m_mounted;
        TQString m_device;
};

inline
bool SQ_MountViewItem::mounted() const
{
    return m_mounted;
}

inline
TQString SQ_MountViewItem::device() const
{
    return m_device;
}

inline
void SQ_MountViewItem::setDevice(const TQString &dev)
{
    m_device = dev;
}

#endif
