/***************************************************************************
                          sq_bcglabel.cpp  -  description
                             -------------------
    begin                : ??? May 4 2005
    copyright            : (C) 2005 by Baryshev Dmitry
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sq_bcglabel.h"

SQ_BCGLabel::SQ_BCGLabel(TQWidget *parent, const char *name, WFlags f) : TQLabel(parent, name, f | TQt::WNoAutoErase)
{
    setScaledContents(false);

    setAlignment(TQt::AlignHCenter | TQt::AlignVCenter);
}

SQ_BCGLabel::~SQ_BCGLabel()
{}
