/***************************************************************************
                          sq_filedialog.h  -  description
                             -------------------
    begin                : Mon Mar 5 2007
    copyright            : (C) 2007 by Baryshev Dmitry
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SQ_FILEDIALOG_H
#define SQ_FILEDIALOG_H

#include <tdefiledialog.h>

/**
  *@author Baryshev Dmitry
  */

class SQ_FileDialog : public KFileDialog
{
    public:
        SQ_FileDialog(const TQString &path, TQWidget *parent);
        ~SQ_FileDialog();

        TQString nameFilter() const;
        void updateCombo(bool enable);

        void setCurrentFilter(const TQString &);
};

#endif
