/***************************************************************************
                          sq_label.h  -  description
                             -------------------
    begin                : June 10 2005
    copyright            : (C) 2005 by Baryshev Dmitry
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SQ_LABEL_H
#define SQ_LABEL_H

#include <tqwidget.h>

class SQ_Label : public TQWidget
{
    public:
        SQ_Label(TQWidget *parent = 0, const char *name = 0);
        ~SQ_Label();

        void setText(const TQString &lt, const TQString &rt);
        void setText(const TQString &lt);
        void setSingle(bool s);

    protected:
        void paintEvent(TQPaintEvent *);

    private:
        TQString ltext, rtext;
        bool single;
};

#endif
