/***************************************************************************
                          sq_popupmenu.cpp  -  description
                             -------------------
    begin                : ??? June 3 2006
    copyright            : (C) 2006 by Baryshev Dmitry
    email                : ksquirrel.iv@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "sq_popupmenu.h"

SQ_PopupMenu::SQ_PopupMenu(TQWidget *parent, const char *name)
    : TDEPopupMenu(parent, name), title(0)
{}

SQ_PopupMenu::~SQ_PopupMenu()
{}

void SQ_PopupMenu::insertTitle(const TQString &t)
{
    title = TDEPopupMenu::insertTitle(t);
}

void SQ_PopupMenu::changeTitle(const TQString &t)
{
    TDEPopupMenu::changeTitle(title, t);
}
